@@ -1,6 +1,7 @@
 from collections import deque
 from collections.abc import Set, Mapping
 from functools import partial, wraps
+from operator import attrgetter
 
 from anygraph.tools import unique_name, save_graph_image
 from anygraph.visitors import Iterator, Visitor
@@ -38,10 +39,10 @@ class BaseDelegate(object):
         self.exclude(*self.targets.values())
 
     def _set(self, target):
-        raise NotImplementedError
+        self.targets[self.get_id(target)] = target
 
     def _del(self, target):
-        raise NotImplementedError
+        self.targets.pop(self.get_id(target), None)
 
 
 class DelegateSet(BaseDelegate, Set):
@@ -59,12 +60,6 @@ class DelegateSet(BaseDelegate, Set):
                 return self.targets[key]
         raise IndexError(str(index))
 
-    def _set(self, target):
-        self.targets[self.get_id(target)] = target
-
-    def _del(self, target):
-        self.targets.pop(self.get_id(target), None)
-
     def __str__(self):
         return f"{{{', '.join(map(str, self))}}}"
 
@@ -74,13 +69,6 @@ class DelegateSet(BaseDelegate, Set):
 
 class DelegateMap(BaseDelegate, Mapping):
 
-    def __init__(self, owner, linker):
-        super().__init__(owner, linker)
-        self.get_key = linker.get_key
-
-    def __len__(self):
-        return len(self.targets)
-
     def __iter__(self):
         return iter(self.targets)
 
@@ -95,27 +83,19 @@ class DelegateMap(BaseDelegate, Mapping):
         if isinstance(old_key_or_target, str):
             old_key = old_key_or_target
         else:
-            old_key = self.find_key(old_key_or_target)
+            old_key = self.get_key(old_key_or_target)
         keys = list(self.targets)
         index = keys.index(old_key)
         below = {k: self.targets.pop(k) for k in keys[index + 1:]}
         self.targets[new_key] = self.targets.pop(old_key)
         self.targets.update(below)
 
-    def find_key(self, target):
+    def get_key(self, target):
         """ find the key of a target """
         for key, targ in self.targets.items():
             if target is targ:
                 return key
 
-    def _set(self, target):
-        self.targets[self.get_key(self.owner, target)] = target
-
-    def _del(self, target):
-        for key, targ in self.targets.items():
-            if target is targ:
-                return self.targets.pop(key, None)
-
     def __str__(self):
         return str(self.targets)
 
@@ -130,9 +110,9 @@ class BaseLinker(object):
     get_id = id  # default
 
     _installables = ('iterate', 'visit', 'build', 'gather', 'gather_pairs', 'find', 'reachable', 'walk',
-                     'endpoints', 'is_cyclic', 'in_cycle', 'shortest_path', 'save_image')
+                     'endpoints', 'is_cyclic', 'in_cycle', 'shortest_path', 'shortest_paths', 'save_image')
 
-    def __init__(self, reverse_name=None, cyclic=True, to_self=True, on_link=None, on_unlink=None, install=False, get_id=None):
+    def __init__(self, reverse_name=None, cyclic=True, to_self=True, on_link=None, on_unlink=None, install=False, get_id=None, **kwargs):
         """
         :param reverse_name: optional name of the reverse relationship
         :param cyclic: whether the graph is allowed to be cyclic
@@ -141,6 +121,7 @@ class BaseLinker(object):
         :param on_unlink(obj, next_obj): optional callback called just before a connection is broken
         :param get_id(obj): optional alternative callback to uniquely identify nodes
         """
+        super().__init__(**kwargs)
         self.reverse_name = reverse_name
         self.cyclic = cyclic
         self.to_self = to_self
@@ -341,6 +322,23 @@ class BaseLinker(object):
                                                  get_cost=get_cost,
                                                  heuristic=heuristic)
 
+    def shortest_paths(self, start_obj, target_objs, get_cost=None, allow_partial=False):
+        """
+         Finds the shortest paths through the graph from start_obj to all target_objs
+        :param start_obj: node to start from
+        :param target_objs: a sequence of nodes to which the paths must be calculated
+        :param get_cost(node, next_node): cost function: must return cost for following edge between node and next_node.
+            Default results in a shortest path defined by the number of edges between start and end
+        :param allow_partial: indicates whether result should be returned even if not all targets are reachable
+        :return: a list of lists of nodes of the shortest paths
+
+        Note that there is no heuristic option, since the heuristic version of the algorithm has no speed advantage for
+            multiple targets (you might as well run the single target version multiple times.
+        """
+        return Iterator(self.name).shortest_paths(start_obj, target_objs,
+                                                  get_cost=get_cost,
+                                                  allow_partial=allow_partial)
+
     def save_image(self, start_obj, filename, label_getter=lambda obj: obj.name,
                    view=False, fontsize='10', fontname='Arial bold', **options):
 
@@ -525,17 +523,11 @@ class Many(BaseMany):
 class ManyMap(BaseMany):
     many_class = DelegateMap
 
-    def __init__(self, *args, key_attr='name', **kwargs):
-        super().__init__(*args, **kwargs)
-        self.key_attr = key_attr
-
-    def get_key(self, obj, target):
-        return unique_name(self.__get__(obj),
-                           target.__class__.__name__.lower(),
-                           getattr(target, self.key_attr, None))
+    def __init__(self, *args, key='name', **kwargs):
+        super().__init__(*args, **kwargs, get_id=attrgetter(key))
 
     def _existing(self, obj, target):
-        return target in self.__get__(obj).values()
+        return self.get_id(target) in self.__get__(obj)
 
 
 if __name__ == '__main__':
